/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
package it.tidalwave.openrdf.elmo;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import org.openrdf.elmo.ElmoManager;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class ElmoManagerThreadLocal
  {
    private ElmoManagerThreadLocal()
      {
      }
    
    private final static ThreadLocal<ElmoManager> tl = new ThreadLocal<ElmoManager>();

    @Nonnull
    public static ElmoManager get()
      throws IllegalStateException
      {
        final ElmoManager em = tl.get();

        if (em == null)
          {
            throw new IllegalStateException("No ElmoManager bound to the current thread: " + Thread.currentThread());
          }

        return em;
      }

    public static void set (@Nullable final ElmoManager elmoManager)
      {
        tl.set(elmoManager);
      }
  }
