/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.tidalwave.openrdf.elmo.impl;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import org.openrdf.concepts.skos.core.Concept;
import it.tidalwave.util.NotFoundException;

/**
 *
 * @author fritz
 */
public final class Utils
  {
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static <T> List<T> filtered (final @Nonnull Set<Concept> broaders, final @Nonnull Class<T> type)
      {
        assert broaders != null : "broaders is null";
        assert type != null : "type is null";

        final List<T> result = new ArrayList<T>();

        for (final Concept concept : broaders)
          {
            if (instanceOf(concept, type))
              {
                result.add((T) concept);
              }
          }

        return result;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static <T> T single (final @Nonnull List<T> broaders)
      throws NotFoundException
      {
        switch (broaders.size())
          {
            case 0:
              throw new NotFoundException();

            case 1:
              return broaders.get(0);

            default:
              throw new RuntimeException("Multiple results found where zero or one are allowed");
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static boolean instanceOf (final @Nonnull Object object, final @Nonnull Class<?> clazz)
      {
        if (clazz.isAssignableFrom(object.getClass()))
          {
            return true;
          }

        for (final Class<?> intf : object.getClass().getInterfaces())
          {
            if (clazz.isAssignableFrom(intf))
              {
                return true;
              }
          }

        return false;
      }
  }
