/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.xml.namespace.QName;
import org.openide.util.Parameters;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.openrdf.elmo.impl.ElmoEntityFactory;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class EntityFactory<T, I extends T>
  {
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    static public interface EntityInitializer<K>
      {
        public void initialize (@Nonnull K entity);
      }

    private static final EntityInitializer<Object> EMPTY_INITIALIZER = new EntityInitializer()
      {
        @Override
        public void initialize (final @Nonnull Object entity)
          {
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static final <I> EntityInitializer<I> emptyInitializer()
      {
        return (EntityInitializer<I>)EMPTY_INITIALIZER;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public final static class Builder<T, I extends T>
      {
        @Nonnull
        private final Class<T> interfaceClass;

        @CheckForNull
        private Class<I> implementationClass;

        @CheckForNull
        private Class<?>[] extraClasses;

        /***************************************************************************************************************
         *
         *
         **************************************************************************************************************/
        protected Builder (final @Nonnull Class<T> interfaceClass)
          {
            Parameters.notNull("interfaceClass", interfaceClass);
            this.interfaceClass = interfaceClass;
          }

        /***************************************************************************************************************
         *
         *
         **************************************************************************************************************/
        @Nonnull
        public Builder<T, I> withImplementation (final @Nonnull Class<I> implementationClass)
          {
            Parameters.notNull("implementationClass", implementationClass);
            this.implementationClass = implementationClass;
            return this;
          }

        /***************************************************************************************************************
         *
         *
         **************************************************************************************************************/
        @Nonnull
        public Builder<T, I> andRoles (final @Nonnull Class<?> ... extraClasses)
          {
            this.extraClasses = extraClasses.clone();
            return this;
          }

        /***************************************************************************************************************
         *
         *
         **************************************************************************************************************/
        @Nonnull
        public EntityFactory<T, I> create()
          {
            return new ElmoEntityFactory(interfaceClass, implementationClass, extraClasses);
//            return new ElmoEntityFactory<T, I>(interfaceClass, implementationClass, extraClasses);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static <T, I extends T> Builder<T, I> forInterface (final @Nonnull Class<T> interfaceClass)
      {
        return new Builder<T, I>(interfaceClass);
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public abstract T create (@Nonnull QName qName);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public abstract T create (@Nonnull QName qName, @Nonnull EntityInitializer<I> initializer);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public abstract T find (@Nonnull QName qName)
      throws NotFoundException;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public abstract T findOrCreate (@Nonnull QName paramQName);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public abstract T findOrCreate (@Nonnull QName paramQName, @Nonnull EntityInitializer<I> paramEntityInitializer);
  }
