/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.tidalwave.semantic;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.lang.ref.WeakReference;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 *
 * @author fritz
 */
public class IdentityWeakHashMap<K, V> implements Map<K, V>
  {
    private static class EntryImpl<K, V> implements Entry<K, V>
      {
        private final K key;
        private final V value;

        public EntryImpl (final K key, final V value)
          {
            this.key = key;
            this.value = value;
          }

        @Override
        public K getKey()
          {
            return key;
          }

        @Override
        public V getValue()
          {
            return value;
          }

        @Override
        public V setValue(V arg0)
          {
            throw new UnsupportedOperationException("Not supported yet.");
          }
      }

    private static class IdentityWeakReference<T> extends WeakReference<T>
      {
        private final int hashCode;

        public IdentityWeakReference (@Nonnull final T referent)
          {
            super(referent);
//            hashCode = referent.hashCode(); // must be immutable
            hashCode = System.identityHashCode(referent); // must be immutable
          }

        @Override
        public boolean equals (@CheckForNull final Object object)
          {
            if (!(object instanceof IdentityWeakReference))
              {
                return false;
              }

            final IdentityWeakReference<T> other = (IdentityWeakReference<T>)object;
            return get() == other.get();
          }

        @Override
        public int hashCode()
          {
            return hashCode;
          }
      }

    private final Map<IdentityWeakReference<K>, IdentityWeakReference<V>> map =
            new HashMap<IdentityWeakReference<K>, IdentityWeakReference<V>>();
            
    public int size() 
      {
        return map.size();
      }

    public boolean isEmpty() 
      {
        return map.isEmpty();
      }

    public boolean containsKey (@Nonnull final Object key)
      {
        return map.containsKey(new IdentityWeakReference<K>((K)key));
      }

    public boolean containsValue (@Nonnull final Object value)
      {
        return map.containsValue(new IdentityWeakReference<V>((V)value));
      }

    public V get (@Nonnull final Object key)
      {
        final IdentityWeakReference<V> reference = map.get(key);
        return (reference != null) ? reference.get() : null;
      }

    public V put (@Nonnull final K key, @Nonnull final V value)
      {
        map.put(new IdentityWeakReference<K>(key), new IdentityWeakReference<V>(value));
        return value;
      }

    public V remove (@Nonnull final Object key)
      {
        final IdentityWeakReference<V> reference = map.remove(new IdentityWeakReference<K>((K) key));
        return (reference != null) ? reference.get() : null;
      }

    public void putAll (@Nonnull final Map<? extends K, ? extends V> m)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public void clear()
      {
        map.clear();
      }

    public Set<K> keySet()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public Collection<V> values()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public Set<Entry<K, V>> entrySet()
      {
        final Set<Entry<K, V>> result = new HashSet<Entry<K, V>>();
        
        for (final Entry<IdentityWeakReference<K>, IdentityWeakReference<V>> entry : map.entrySet())
          {
            result.add(new EntryImpl<K, V>(entry.getKey().get(), entry.getValue().get()) {});
          }

        return result;
      }


    @Override
    @Nonnull
    public String toString()
      {
        final StringBuilder builder = new StringBuilder();
        builder.append("[");
        String separator = "";

        for (final Entry<K, V> entry : entrySet())
          {
            builder.append(String.format("%s%s -> %s", separator, entry.getKey(), entry.getValue()));
            separator = ", ";
          }

        builder.append("]");
        return builder.toString();
      }
  }
